﻿Imports Grasshopper.Kernel
Imports Grasshopper.Kernel.Types
Imports Grasshopper.Kernel.Data
Imports System.Math
'Imports Rhino.Geometry

Public Class Class1
    Inherits GH_Component

    ''constructor
    Public Sub New()

        MyBase.New("Sap2000 - Testna komponenta", "Most", "Primer prenosa modela v program Sap2000, analize in prenosa rezultatov", "PrimeriKomponent_ZigaSebenik", "Most")
    End Sub

    Public Overrides ReadOnly Property ComponentGuid As Guid
        Get
            Return New Guid("8fa16c7b-b160-4a23-b445-a9ce33364866")
        End Get
    End Property

    Protected Overrides Sub RegisterInputParams(pManager As Grasshopper.Kernel.GH_Component.GH_InputParamManager)
        pManager.AddGenericParameter("Koordinate vozlisc ploskovnih KE", "P_KE", "generic_parameter", GH_ParamAccess.tree)
        pManager.AddNumberParameter("Ploskovna obtezba [kN/m2]", "O", "number_parameter", GH_ParamAccess.item)
        pManager.AddGenericParameter("Koordinate vozlisc linijskih KE", "L_KE", "generic_parameter", GH_ParamAccess.tree)
        pManager.AddGenericParameter("Koordinate podprtih tock", "Pod", "generic_parameter", GH_ParamAccess.tree)
        pManager.AddGenericParameter("Koordinate opazovane tocke", "T", "generic_parameter", GH_ParamAccess.tree)
    End Sub

    Protected Overrides Sub RegisterOutputParams(pManager As Grasshopper.Kernel.GH_Component.GH_OutputParamManager)
        pManager.AddNumberParameter("Pomik [m]", "Uz", "number_parameter", GH_ParamAccess.item)
    End Sub

    Protected Overrides Sub SolveInstance(DA As IGH_DataAccess)
        Dim list As New GH_Structure(Of IGH_Goo)
        Dim obtezba As Double
        Dim frameElementi As GH_Structure(Of IGH_Goo)
        Dim podpore As GH_Structure(Of IGH_Goo)
        Dim tockaPomika As GH_Structure(Of IGH_Goo)

        DA.GetDataTree(Of IGH_Goo)(0, list)
        DA.GetData(Of Double)(1, obtezba)
        DA.GetDataTree(Of IGH_Goo)(2, frameElementi)
        DA.GetDataTree(Of IGH_Goo)(3, podpore)
        DA.GetDataTree(Of IGH_Goo)(4, tockaPomika)

        Dim SapObject As New SAP2000v15.SapObject
        Dim SapModel As New SAP2000v15.cSapModel
        Dim ret As Long

        'Pobrisi 
        ret = SapModel.SetModelIsLocked(False)
        ret = SapModel.AreaObj.Delete("areaGroup", SAP2000v15.eItemType.Group)
        ret = SapModel.FrameObj.Delete("frameGroup", SAP2000v15.eItemType.Group)
        ret = SapModel.SelectObj.All(False)
        ret = SapModel.PointObj.DeleteSpecialPoint("ignored", SAP2000v15.eItemType.SelectedObjects)
        ret = SapModel.LoadCases.Delete("StaticLoadCase")
        ret = SapModel.LoadPatterns.Delete("Obtezba")
        ret = SapModel.PropFrame.Delete("prerezFrame")
        ret = SapModel.PropArea.Delete("prerezArea")
        ret = SapModel.PropMaterial.Delete("Beton")
        ret = SapModel.PropMaterial.Delete("Jeklo")
        ret = SapModel.RespCombo.Delete("COMB1")

        'Add group
        ret = SapModel.GroupDef.SetGroup("areaGroup")
        ret = SapModel.GroupDef.SetGroup("frameGroup")

        'Material
        ret = SapModel.PropMaterial.SetMaterial("Beton", SAP2000v15.eMatType.MATERIAL_CONCRETE)
        ret = SapModel.PropMaterial.SetMPIsotropic("Beton", 33000000, 0.2, 0.00001)
        ret = SapModel.PropMaterial.SetMaterial("Jeklo", SAP2000v15.eMatType.MATERIAL_STEEL)
        ret = SapModel.PropMaterial.SetMPIsotropic("Jeklo", 210000000, 0.3, 0.00001)

        'Prerezi
        ret = SapModel.PropFrame.SetPipe("prerezFrame", "Jeklo", 0.15, 0.005)
        ret = SapModel.PropArea.SetShell_1("prerezArea", 2, True, "Beton", 0, 0.3, 0.3)

        'Ploskovni elemeti
        Dim tocka() As GH_Point

        ret = SapModel.SetPresentUnits(SAP2000v15.eUnits.kN_m_C)

        For i = 0 To list.Branches.Count - 1
            Dim countB As Integer = list.Branch(i).Count
            ReDim tocka(countB)
            Dim X(countB - 1) As Double
            Dim Y(countB - 1) As Double
            Dim Z(countB - 1) As Double
            For j = 0 To countB - 1
                tocka(j) = list.Branch(i).Item(j)
                X(j) = tocka(j).Value.X
                Y(j) = tocka(j).Value.Y
                Z(j) = tocka(j).Value.Z
            Next
            ret = SapModel.AreaObj.AddByCoord(countB, X, Y, Z, "Name", "prerezArea")
        Next

        'Frame elementi & frameGroup
        For i = 0 To frameElementi.Branches.Count - 1
            Dim countB As Integer = frameElementi.Branch(i).Count
            ReDim tocka(countB - 1)
            Dim X(countB - 1) As Double
            Dim Y(countB - 1) As Double
            Dim Z(countB - 1) As Double
            For j = 0 To countB - 1
                tocka(j) = frameElementi.Branch(i).Item(j)
                X(j) = tocka(j).Value.X
                Y(j) = tocka(j).Value.Y
                Z(j) = tocka(j).Value.Z
            Next
            ret = SapModel.FrameObj.AddByCoord(X(0), Y(0), Z(0), X(1), Y(1), Z(1), "Name", "prerezFrame")
        Next

        Dim NumberNames As Long
        Dim MyName() As String
        ret = SapModel.FrameObj.GetNameList(NumberNames, MyName)
        For i = 0 To NumberNames - 1
            ret = SapModel.FrameObj.SetGroupAssign(MyName(i), "frameGroup")
        Next

        'Podpore
        ReDim tocka(0)
        Dim podpiranje(5) As Boolean
        For i = 0 To 2
            podpiranje(i) = True
        Next
        For i = 3 To 5
            podpiranje(i) = False
        Next
        For i = 0 To 3
            tocka(0) = podpore.ElementAt(i)
            ret = SapModel.PointObj.AddCartesian(tocka(0).Value.X, tocka(0).Value.Y, tocka(0).Value.Z, "name", "P" & CStr(i))
            If i = 1 Or i = 3 Then
                podpiranje(0) = False
            Else
                podpiranje(0) = True
            End If
            ret = SapModel.PointObj.SetRestraint("P" & CStr(i), podpiranje)
            ret = SapModel.PointObj.SetSpecialPoint("P" & CStr(i), True)
        Next

        'Tocka opazovanega pomika
        ReDim tocka(0)
        tocka(0) = tockaPomika.ElementAt(0)
        ret = SapModel.PointObj.AddCartesian(tocka(0).Value.X, tocka(0).Value.Y, tocka(0).Value.Z, "name", "T")
        ret = SapModel.PointObj.SetSpecialPoint("T", True)

        'Obtezba & areaGroup
        ret = SapModel.AreaObj.GetNameList(NumberNames, MyName)

        ret = SapModel.LoadPatterns.Add("Obtezba", SAP2000v15.eLoadPatternType.LTYPE_LIVE)

        For i = 0 To NumberNames - 1
            ret = SapModel.AreaObj.SetLoadUniform(MyName(i), "Obtezba", obtezba, 10, False, , SAP2000v15.eItemType.Object)
            ret = SapModel.AreaObj.SetGroupAssign(MyName(i), "areaGroup")
        Next

        'Load Case
        Dim MyLoadType() As String = {"Load"}
        Dim MyLoadName() As String = {"Obtezba"}
        Dim MySF() As Double = {1}

        ret = SapModel.LoadCases.StaticLinear.SetCase("StaticLoadCase")
        ret = SapModel.LoadCases.StaticLinear.SetLoads("StaticLoadCase", 1, MyLoadType, MyLoadName, MySF)

        'Load Combination
        ret = SapModel.RespCombo.Add("COMB1", 1)
        ret = SapModel.RespCombo.SetCaseList("COMB1", 0, "DEAD", 1.35)
        ret = SapModel.RespCombo.SetCaseList("COMB1", 0, "StaticLoadCase", 1.5)

        'Save and Start Analysis
        Dim tempPath As String = System.IO.Path.GetTempPath
        ret = SapModel.Analyze.SetRunCaseFlag("MODAL", False)
        ret = SapModel.File.Save(tempPath + "\Example_by_ZigaSebenik.sdb")
        ret = SapModel.Analyze.RunAnalysis

        'Get Results
        Dim ObjectElm As String
        Dim NumberResults As Integer
        Dim Obj() As String
        Dim Elm() As String
        Dim LoadCase() As String
        Dim StepType() As String
        Dim StepNum() As Double
        Dim U1() As Double
        Dim U2() As Double
        Dim U3() As Double
        Dim R1() As Double
        Dim R2() As Double
        Dim R3() As Double


        ret = SapModel.Results.Setup.DeselectAllCasesAndCombosForOutput
        ret = SapModel.Results.Setup.SetComboSelectedForOutput("COMB1")
        ret = SapModel.Results.JointDispl("T", ObjectElm, NumberResults, Obj, Elm, LoadCase, StepType, StepNum, U1, U2, U3, R1, R2, R3)

        DA.SetData(0, U3(1)) 'Return results

        'Clear Selection
        ret = SapModel.SelectObj.ClearSelection

    End Sub

End Class